@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2006, 2008 Proofpoint, Inc. and its suppliers.
+ * Copyright (c) 1998-2006, 2008, 2023, 2024 Proofpoint, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -301,9 +301,9 @@ incbuflen(buflen)
 /* values for input state machine */
 #define IS_NORM		0	/* middle of line */
 #define IS_BOL		1	/* beginning of line */
-#define IS_DOT		2	/* read a dot at beginning of line */
+#define IS_DOT		2	/* read "." at beginning of line */
 #define IS_DOTCR	3	/* read ".\r" at beginning of line */
-#define IS_CR		4	/* read a carriage return */
+#define IS_CR		4	/* read "\r" */
 
 /* hack to enhance readability of debug output */
 static const char *istates[] = { "NORM", "BOL", "DOT", "DOTCR", "CR" };
@@ -314,6 +314,8 @@ static const char *istates[] = { "NORM", "BOL", "DOT", "DOTCR", "CR" };
 #define MS_HEADER	1	/* reading message header */
 #define MS_BODY		2	/* reading message body */
 #define MS_DISCARD	3	/* discarding rest of message */
+#define BARE_LF_MSG "Bare linefeed (LF) not allowed"
+#define BARE_CR_MSG "Bare carriage return (CR) not allowed"
 
 void
 collect(fp, smtpmode, hdrp, e, rsetsize)
@@ -344,15 +346,13 @@ collect(fp, smtpmode, hdrp, e, rsetsize)
 #if _FFR_REJECT_NUL_BYTE
 	bool hasNUL;		/* has at least one NUL input byte */
 #endif
-#if _FFR_BARE_LF
 	bool bare_lf = false;
-#endif
+	bool bare_cr = false;
 
 #define SMTPMODE	(smtpmode >= SMTPMODE_LAX)
 #define SMTPMODE_STRICT	((smtpmode & SMTPMODE_CRLF) != 0)
-#if _FFR_BARE_LF
-# define SMTPMODE_NO_BARE_LF	((smtpmode & SMTPMODE_LF) != 0)
-#endif
+#define SMTPMODE_BARE_LF_421	((smtpmode & SMTPMODE_LF_421) != 0)
+#define SMTPMODE_BARE_CR_421	((smtpmode & SMTPMODE_CR_421) != 0)
 
 	df = NULL;
 	ignrdot = SMTPMODE ? false : IgnrDot;
@@ -382,14 +382,14 @@ collect(fp, smtpmode, hdrp, e, rsetsize)
 	*/
 
 	if (SMTPMODE)
-		message("354 Enter mail, end with \".\" on a line by itself");
+		message("354 End data with <CR><LF>.<CR><LF>");
 
 	/* simulate an I/O timeout when used as sink */
 	if (tTd(83, 101))
 		sleep(319);
 
 	if (tTd(30, 2))
-		sm_dprintf("collect\n");
+		sm_dprintf("collect, smtpmode=%#x\n", smtpmode);
 
 	/*
 	**  Read the message.
@@ -474,11 +474,27 @@ collect(fp, smtpmode, hdrp, e, rsetsize)
 			if (tTd(30, 94))
 				sm_dprintf("istate=%s, c=%c (0x%x)\n",
 					ISTATE, (char) c, c);
-#if _FFR_BARE_LF
-			if ('\n' == c && SMTPMODE_NO_BARE_LF &&
+			if ('\n' == c && !bare_lf && SMTPMODE &&
 			    !(IS_CR == istate || IS_DOTCR == istate))
+			{
 				bare_lf = true;
-#endif
+				if (SMTPMODE_BARE_LF_421)
+				{
+					inputerr = true;
+					goto readabort;
+				}
+			}
+			if (!bare_cr && SMTPMODE &&
+			    (IS_CR == istate || IS_DOTCR == istate)
+			    && '\n' != c)
+			{
+				bare_cr = true;
+				if (SMTPMODE_BARE_CR_421)
+				{
+					inputerr = true;
+					goto readabort;
+				}
+			}
 			switch (istate)
 			{
 			  case IS_BOL:
@@ -870,21 +886,26 @@ readdone:
 			problem = "unexpected close";
 		else if (sm_io_error(fp))
 			problem = "I/O error";
+		else if (bare_lf)
+			problem = "bare LF";
+		else if (bare_cr)
+			problem = "bare CR";
 		else
 			problem = "read timeout";
-		if (LogLevel > 0 && sm_io_eof(fp))
+
+#define CONN_ERR_TXT	"collect: %s on connection from %s, from=%s%s%s"
+#define CONN_ERR_CODE	"421 4.4.1 "
+#define CONN_ERR_BARE bare_lf ? BARE_LF_MSG : (bare_cr ? BARE_CR_MSG : "")
+#define CONN_ERR_ARGS problem, host, shortenstring(e->e_from.q_paddr, MAXSHORTSTR), (bare_lf || bare_cr) ? ", info=" : "", CONN_ERR_BARE
+		if (LogLevel > 0 && (sm_io_eof(fp) || bare_lf || bare_cr))
 			sm_syslog(LOG_NOTICE, e->e_id,
-				"collect: %s on connection from %.100s, sender=%s",
-				problem, host,
-				shortenstring(e->e_from.q_paddr, MAXSHORTSTR));
-		if (sm_io_eof(fp))
-			usrerr("421 4.4.1 collect: %s on connection from %s, from=%s",
-				problem, host,
-				shortenstring(e->e_from.q_paddr, MAXSHORTSTR));
+				CONN_ERR_TXT, CONN_ERR_ARGS);
+		if (bare_lf || bare_cr)
+			usrerr(CONN_ERR_CODE "%s", CONN_ERR_BARE);
+		else if (sm_io_eof(fp))
+			usrerr(CONN_ERR_CODE CONN_ERR_TXT, CONN_ERR_ARGS);
 		else
-			syserr("421 4.4.1 collect: %s on connection from %s, from=%s",
-				problem, host,
-				shortenstring(e->e_from.q_paddr, MAXSHORTSTR));
+			syserr(CONN_ERR_CODE CONN_ERR_TXT, CONN_ERR_ARGS);
 		flush_errors(true);
 
 		/* don't return an error indication */
@@ -914,21 +935,10 @@ readdone:
 		logsender(e, e->e_msgid);
 		e->e_flags &= ~EF_LOGSENDER;
 	}
-#if _FFR_BARE_LF
-	if (bare_lf)
-	{
-		e->e_flags |= EF_NO_BODY_RETN|EF_CLRQUEUE;
-		if (!bitset(EF_FATALERRS, e->e_flags))
-		{
-			e->e_status = "5.6.0";
-			usrerrenh(e->e_status,
-				"550 Bare linefeed (LF) not allowed");
-			if (LogLevel > 10)
-				sm_syslog(LOG_NOTICE, e->e_id,
-					"Bare linefeed (LF) not allowed");
-		}
-	}
-#endif /* _FFR_BARE_LF */
+	if (bare_lf && LogLevel > 8)
+		sm_syslog(LOG_NOTICE, e->e_id, BARE_LF_MSG);
+	if (bare_cr && LogLevel > 8)
+		sm_syslog(LOG_NOTICE, e->e_id, BARE_CR_MSG);
 
 	/* check for message too large */
 	if (bitset(EF_TOOBIG, e->e_flags))
