@@ -347,7 +347,7 @@ mime8to7(mci, header, e, boundaries, flags, level)
 				goto writeerr;
 			if (tTd(43, 35))
 				sm_dprintf("  ...%s\n", buf);
-			collect(e->e_dfp, false, &hdr, e, false);
+			collect(e->e_dfp, SMTPMODE_NO, &hdr, e, false);
 			if (tTd(43, 101))
 				putline("+++after collect", mci);
 			if (!putheader(mci, hdr, e, flags))
@@ -409,7 +409,7 @@ mime8to7(mci, header, e, boundaries, flags, level)
 				goto writeerr;
 
 			mci->mci_flags |= MCIF_INMIME;
-			collect(e->e_dfp, false, &hdr, e, false);
+			collect(e->e_dfp, SMTPMODE_NO, &hdr, e, false);
 			if (tTd(43, 101))
 				putline("+++after collect", mci);
 			if (!putheader(mci, hdr, e, flags))
