@@ -448,7 +448,7 @@ allocaddr(a, flags, paddr, e)
 	ENVELOPE *e;
 {
 	if (tTd(24, 4))
-		sm_dprintf("allocaddr(flags=%x, paddr=%s, ad=%d)\n", flags, paddr, bitset(EF_SECURE, e->e_flags));
+		sm_dprintf("allocaddr: flags=%x, paddr=%s, ad=%d\n", flags, paddr, bitset(EF_SECURE, e->e_flags));
 
 	a->q_paddr = paddr;
 
@@ -689,7 +689,6 @@ unsigned char	TokTypeNoC[256] =
 	ATM,ATM,ATM,ATM,ATM,ATM,ATM,ATM, ATM,ATM,ATM,ATM,ATM,ATM,ATM,ONE
 };
 
-
 #define NOCHAR		(-1)	/* signal nothing in lookahead token */
 
 char **
@@ -2495,6 +2494,7 @@ static struct qflags	AddressFlags[] =
 	{ "QSECURE",		QSECURE		},
 	{ "QQUEUED",		QQUEUED		},
 	{ "QINTREPLY",		QINTREPLY	},
+	{ "QMXSECURE",		QMXSECURE	},
 	{ "QTHISPASS",		QTHISPASS	},
 	{ "QRCPTOK",		QRCPTOK		},
 	{ NULL,			0		}
